@@ -1,4 +1,4 @@
-/////////////////////////////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////////////////////
 //                                                                                 //
 //    KGGZ - The KDE client for the GGZ Gaming Zone - Version 0.0.5pre             //
 //    Copyright (C) 2000 - 2002 Josef Spillner - dr_maux@users.sourceforge.net     //
@@ -51,10 +51,17 @@
 #include <klocale.h>
 #include <kurl.h>
 #include <krandomsequence.h>
+#ifdef KGGZ_WALLET
+#include <klineeditdlg.h>
+#include <kwallet.h>
+#endif
+#include <ksimpleconfig.h>
+#include <kapplication.h>
 
 // Qt includes
 #include <qiconview.h>
 #include <qlayout.h>
+#include <qdir.h>
 
 // System includes
 #include <stdio.h>
@@ -75,10 +82,6 @@ KGGZ::KGGZ(QWidget *parent, const char *name)
 	m_gamefd = -1;
 	kggzroom = NULL;
 	kggzserver = NULL;
-	m_save_username = NULL;
-	m_save_password = NULL;
-	m_save_host = NULL;
-	m_save_hostname = NULL;
 	m_launch = NULL;
 	kggzgame = NULL;
 #ifdef KGGZ_BROWSER
@@ -109,7 +112,8 @@ KGGZ::KGGZ(QWidget *parent, const char *name)
 
 	m_workspace = new KGGZWorkspace(this, "workspace");
 
-	connect(m_workspace->widgetChat(), SIGNAL(signalChat(const char *, char *, int)), SLOT(slotChat(const char *, char *, int)));
+	connect(m_workspace->widgetChat(), SIGNAL(signalChat(QString, QString, int)), SLOT(slotChat(QString, QString, int)));
+	connect(m_workspace->widgetUsers(), SIGNAL(signalChat(QString, QString, int)), SLOT(slotChat(QString, QString, int)));
 	connect(m_workspace->widgetLogo(), SIGNAL(signalInfo()), SLOT(menuGameInfo()));
 
 	m_core = new GGZCore();
@@ -121,6 +125,7 @@ KGGZ::KGGZ(QWidget *parent, const char *name)
 	}
 
 	m_config = NULL;
+	m_connfront = false;
 	
 	readConfiguration(true);
 	
@@ -152,7 +157,8 @@ void KGGZ::readConfiguration(bool immediate)
 	if(m_config) delete m_config;
 	m_config = new GGZCoreConf();
 
-	result = m_config->init(KGGZ_DIRECTORY "/kggzrc", QString("%1/.ggz/kggz.rc").arg(getenv("HOME")).latin1());
+	result = m_config->init(KGGZ_DIRECTORY "/kggzrc",
+			(QDir::homeDirPath() + "/.ggz/kggz.rc").utf8());
 	if(result == -1)
 	{
 		KGGZDEBUG("Critical: Could not open configuration file!\n");
@@ -202,7 +208,7 @@ void KGGZ::autoconnect(QString uri)
 		mode = GGZCoreServer::guest;
 	}
 
-	slotConnected(url.host().latin1(), (url.port() ? url.port() : 5688), user.latin1(), password.latin1(), mode);
+	slotConnected(url.host(), (url.port() ? url.port() : 5688), user, password, mode);
 }
 
 void KGGZ::resizeEvent(QResizeEvent *e)
@@ -210,8 +216,39 @@ void KGGZ::resizeEvent(QResizeEvent *e)
 	m_workspace->resize(e->size());
 }
 
-void KGGZ::slotConnected(const char *host, int port, const char *username, const char *password, int mode)
+void KGGZ::slotConnected(QString host, int port, QString username, QString password, int mode)
 {
+#ifdef KGGZ_WALLET
+	QString p, entry;
+	KWallet::Wallet *w = NULL;
+
+	if(mode == GGZCoreServer::normal)
+	{
+		w = KWallet::Wallet::openWallet("kggz");
+		if(w)
+		{
+			entry = QString("%1@%2").arg(username).arg(host);
+			w->setFolder("passwords");
+			w->readPassword(entry, p);
+
+			/*KMessageBox::information(this,
+				i18n("Wallet password: %1").arg(password), i18n("debug"));*/
+		}
+		else KMessageBox::error(this,
+			i18n("The wallet could not be opened."), i18n("Connection"));
+
+		if(p.isEmpty())
+		{
+			p = KLineEditDlg::getText(i18n("Password"),
+				i18n("Password not found, please input:"), NULL, NULL);
+			if((p) && (w))
+				w->writePassword(entry, p);
+		}
+		password = p;
+
+	}
+#endif
+
 	if(m_connect->optionServer())
 	{
 		KGGZDEBUG("Start server\n");
@@ -227,17 +264,18 @@ void KGGZ::slotConnected(const char *host, int port, const char *username, const
 	}
 
 	KGGZDEBUG("Connect with: host=%s port=%i username=%s password=%s mode=%i encryption=%i\n",
-		host, port, username, password, mode, m_connect->optionSecure());
+		host.utf8().data(), port, username.utf8().data(),
+		password.utf8().data(), mode, m_connect->optionSecure());
 
 	if(m_dns) delete m_dns;
 	m_dns = new QDns(host, QDns::A);
 	connect(m_dns, SIGNAL(resultsReady()), SLOT(slotConnectedStart()));
 
 	m_save_loginmode = mode;
-	m_save_username = strdup(username);
-	m_save_password = strdup(password);
-	m_save_host = strdup(host);
-	m_save_hostname = strdup(host);
+	m_save_username = username;
+	m_save_password = password;
+	m_save_host = host;
+	m_save_hostname = host;
 	m_save_port = port;
 	m_save_encryption = m_connect->optionSecure();
 }
@@ -256,13 +294,12 @@ void KGGZ::slotConnectedStart()
 	}
 	list = m_dns->addresses();
 	addr = (*list.at(0));
-	if(m_save_host) free(m_save_host);
-	m_save_host = strdup(addr.toString().latin1());
-	KGGZDEBUG("Host resolved to: %s\n", m_save_host);
+	m_save_host = addr.toString();
+	KGGZDEBUG("Host resolved to: %s\n", m_save_host.utf8().data());
 
 	kggzserver = new GGZCoreServer();
 	attachServerCallbacks();
-	kggzserver->logSession(QString("%1/.ggz/kggz.xml-log").arg(getenv("HOME")).latin1());
+	kggzserver->logSession(QString("%1/.ggz/kggz.xml-log").arg(QDir::homeDirPath()).utf8());
 
 	kggzserver->setHost(m_save_host, m_save_port, m_save_encryption);
 	result = kggzserver->connect();
@@ -307,7 +344,8 @@ void KGGZ::menuServerLaunch()
 	GGZCoreConfio *config;
 	char *process;
 
-	config = new GGZCoreConfio(QString("%1/.ggz/kggz.rc").arg(getenv("HOME")).latin1(), GGZCoreConfio::readwrite | GGZCoreConfio::create);
+	config = new GGZCoreConfio(QString("%1/.ggz/kggz.rc").arg(QDir::homeDirPath()).utf8(),
+		GGZCoreConfio::readwrite | GGZCoreConfio::create);
 	process = config->read("Environment", "Server", "/usr/bin/ggzd");
 	delete config;
 
@@ -367,11 +405,12 @@ void KGGZ::menuConnect()
 {
 	if(!m_connect)
 	{
-		m_connect = new KGGZConnect(NULL, "connect");
-		connect(m_connect, SIGNAL(signalConnect(const char*, int, const char*, const char*, int)),
-			SLOT(slotConnected(const char*, int, const char*, const char*, int)));
+		m_connect = new KGGZConnect(this, "connect");
+		connect(m_connect, SIGNAL(signalConnect(QString, int, QString, QString, int)),
+			SLOT(slotConnected(QString, int, QString, QString, int)));
 	}
 	m_connect->show();
+	m_connfront = true;
 }
 
 void KGGZ::menuMotd()
@@ -392,15 +431,18 @@ void KGGZ::dispatcher()
 	if(m_module) delete m_module;
 	if(m_config) delete m_config;
 	if(m_core) delete m_core;
-	if(m_save_username) delete m_save_username;
-	if(m_save_password) delete m_save_password;
-	if(m_save_host) delete m_save_host;
-	if(m_save_hostname) delete m_save_hostname;
 }
 
 void KGGZ::timerEvent(QTimerEvent *e)
 {
-	//static int lag = 0;
+	Q_UNUSED(e);
+
+	if(m_connect)
+		if(m_connfront)
+		{
+			m_connect->raise();
+			m_connfront = false;
+		}
 
 	if(m_channelfd >= 0)
 	{
@@ -429,18 +471,6 @@ void KGGZ::timerEvent(QTimerEvent *e)
 		m_killserver = 0;
 	}
 	if(!kggzserver) return;
-
-	//lag++;
-	//if(lag == 20)
-	//{
-	//	lag = 0;
-	//	lagPlayers();
-	//}
-
-	//if(kggzserver)
-	//{
-	//	if(kggzserver->dataPending()) kggzserver->dataRead();
-	//}
 }
 
 void KGGZ::slotServerData()
@@ -460,8 +490,8 @@ void KGGZ::listTables()
 	GGZCoreTable *table;
 	GGZCoreGametype *gametype;
 	int tableseats, tableseatsopen;
-	const char *tabledescription;
-	const char *playername;
+	QString playername, tabledescription;
+	QString name;
 
 	if(!kggzroom)
 	{
@@ -514,22 +544,23 @@ void KGGZ::listTables()
 			}
 
 			if(table->playerType(j) == GGZ_SEAT_OPEN
-			   || table->playerType(j) == GGZ_SEAT_RESERVED) tableseatsopen++;
+				|| table->playerType(j) == GGZ_SEAT_RESERVED) tableseatsopen++;
 			else
 			{
 				if(table->playerType(j) == GGZ_SEAT_PLAYER)
 				{
 					playername = table->playerName(j);
-					if(strlen(playername) == 0)
+					if(playername.isEmpty())
 					{
-						playername = i18n("(unknown)").latin1();
+						playername = i18n("(unknown)");
 						tableseatsopen++;
 					}
 					m_workspace->widgetUsers()->addTablePlayer(i, playername);
 				}
 			}
 		}
-		m_workspace->widgetTables()->add(gametype->name(), tabledescription, tableseats - tableseatsopen, tableseats);
+		m_workspace->widgetTables()->add(gametype->name(),
+			QString::fromUtf8(tabledescription), tableseats - tableseatsopen, tableseats);
 	}
 }
 
@@ -553,22 +584,25 @@ void KGGZ::listPlayers()
 		switch(player->type())
 		{
 			case GGZCorePlayer::guest:
-				type = KGGZUsers::assignplayer;
+				type = KGGZUsers::assignguest;
 				break;
 			case GGZCorePlayer::normal:
-				type = KGGZUsers::assignbuddy;
+				type = KGGZUsers::assignplayer;
 				break;
 			case GGZCorePlayer::admin:
 				type = KGGZUsers::assignadmin;
 				break;
-			case GGZCorePlayer::none:
-			default:
+			case GGZCorePlayer::bot:
 				type = KGGZUsers::assignbot;
 				break;
+			case GGZCorePlayer::unknown:
+			default:
+				type = KGGZUsers::assignunknown;
+				break;
 		}
 		m_workspace->widgetUsers()->assignRole(playername, type);
 		m_workspace->widgetChat()->chatline()->addPlayer(playername);
-		if(m_grubby) m_grubby->addPlayer(playername);
+		if((m_grubby) && (type == KGGZUsers::assignbot)) m_grubby->addPlayer(playername);
 	}
 	m_workspace->widgetUsers()->assignSelf(m_save_username);
 }
@@ -590,13 +624,15 @@ void KGGZ::lagPlayers()
 	}
 }
 
-void KGGZ::gameCollector(unsigned int id, void* data)
+void KGGZ::gameCollector(unsigned int id, const void* data)
 {
 	int result;
 	int seats;
-	char *description;
+	QString description;
 	GGZCoreGametype *gametype;
 
+	Q_UNUSED(data);
+
 	switch(id)
 	{
 		case GGZCoreGame::launched:
@@ -616,25 +652,6 @@ void KGGZ::gameCollector(unsigned int id, void* data)
 		case GGZCoreGame::negotiatefail:
 			KGGZDEBUG("negotiatefail\n");
 			break;
-		case GGZCoreGame::over:
-			KGGZDEBUG("over\n");
-			delete m_sn_game;
-			m_sn_game = NULL;
-			m_gamefd = -1;
-			eventLeaveTable(1);
-			m_workspace->widgetChat()->receive(NULL, i18n("Game over"), KGGZChat::RECEIVE_ADMIN);
-			emit signalMenu(MENUSIG_GAMEOVER); // for spectators
-			break;
-		case GGZCoreGame::ioerror:
-			KGGZDEBUG("ioerror\n");
-			eventLeaveTable(1);
-			m_workspace->widgetChat()->receive(NULL, i18n("ERROR: Network error!"), KGGZChat::RECEIVE_ADMIN);
-			break;
-		case GGZCoreGame::protoerror:
-			KGGZDEBUG("protoerror\n");
-			eventLeaveTable(1);
-			m_workspace->widgetChat()->receive(NULL, i18n("ERROR: Protocol error!"), KGGZChat::RECEIVE_ADMIN);
-			break;
 		case GGZCoreGame::playing:
 			KGGZDEBUG("playing\n");
 			if(m_gameinfo->type() == KGGZGameInfo::typelaunch)
@@ -651,11 +668,12 @@ void KGGZ::gameCollector(unsigned int id, void* data)
 					KGGZDEBUG("Huh?\n");
 					return;
 				}
-				description = (char*)m_launch->description();
+				if(m_launch->description().isEmpty()) description = "";
+				else description = m_launch->description();
 				seats = m_launch->seats();
 
 				m_table = new GGZCoreTable();
-				m_table->init(gametype->gametype(), description, seats);
+				m_table->init(gametype->gametype(), description.utf8(), seats);
 				for(int i = 0; i < seats; i++)
 				{
 					switch(m_launch->seatType(i))
@@ -667,18 +685,21 @@ void KGGZ::gameCollector(unsigned int id, void* data)
 							KGGZDEBUG("* %i: open\n", i);
 							break;
 						case KGGZLaunch::seatbot:
-							KGGZDEBUG("* %i: bot\n", i);
-							m_table->addBot(NULL, i);
+							KGGZDEBUG("* %i: bot (%s)\n", i, m_launch->reservation(i).utf8().data());
+							if(m_launch->reservation(i).isNull())
+								m_table->addBot(NULL, i);
+							else
+								m_table->addBot(m_launch->reservation(i).utf8(), i);
 							break;
 						case KGGZLaunch::seatreserved:
-							KGGZDEBUG("* %i: reserved\n", i);
-							m_table->addReserved((char*)m_launch->reservation(i).latin1(), i);
+							KGGZDEBUG("* %i: reserved (%s)\n", i, m_launch->reservation(i).utf8().data());
+							m_table->addReserved(m_launch->reservation(i).utf8(), i);
 							break;
 						case KGGZLaunch::seatunused:
 							KGGZDEBUG("* %i: unused.\n", i);
 							break;
 						case KGGZLaunch::seatunknown:
-							default:
+						default:
 							KGGZDEBUG("* %i: unknown!\n", i);
 					}
 				}
@@ -723,11 +744,24 @@ void KGGZ::gameCollector(unsigned int id, void* data)
 	}
 }
 
-void KGGZ::roomCollector(unsigned int id, void* data)
+void KGGZ::roomCollector(unsigned int id, const void* data)
 {
-	const char *chatsender = NULL, *chatmessage = NULL;
+	QString chatsender, chatmessage;
 	int chattype = GGZCoreRoom::chatnormal;
 	QString buffer;
+	GGZRoomChangeEventData *event;
+	QString toroom;
+	QString fromroom;
+	int roomnumber;
+	QString player;
+	GGZRoom *room;
+
+	roomnumber = 0;
+
+	if(id == GGZCoreRoom::chatevent)
+		emit signalActivity(ACTIVITY_DIRECT);
+	else if(id != GGZCoreRoom::playerlag)
+		emit signalActivity(ACTIVITY_ROOM);
 
 	switch(id)
 	{
@@ -735,13 +769,20 @@ void KGGZ::roomCollector(unsigned int id, void* data)
 			chattype = ((GGZChatEventData*)data)->type;
 			chatsender = ((GGZChatEventData*)data)->sender;
 			chatmessage = ((GGZChatEventData*)data)->message;
-			KGGZDEBUG("Chat receives: %s from %s\n", chatmessage, chatsender);
+			KGGZDEBUG("Chat receives: %s from %s\n",
+				chatmessage.utf8().data(), chatsender.utf8().data());
 			if((!m_workspace) || (!m_workspace->widgetChat()))
 			{
 				KGGZDEBUG("Critical: Workspace/Chat absent!\n");
 				return;
 			}
 			break;
+		case GGZCoreRoom::stats:
+			player = QString((char*)data);
+			break;
+		case GGZCoreRoom::count:
+			roomnumber = *(int*)data;
+			break;
 	}
 
 	switch(id)
@@ -759,17 +800,18 @@ void KGGZ::roomCollector(unsigned int id, void* data)
 			{
 				case GGZCoreRoom::chatnormal:
 					KGGZDEBUG("chatnormal\n");
-					if(strncmp(chatmessage, "/me ", 4) == 0)
+					if(chatmessage.startsWith("/me "))
 					{
 						m_workspace->widgetChat()->receive(chatsender, chatmessage, KGGZChat::RECEIVE_ME);
 					}
 					else
 					{
-						if(strcmp(chatsender, m_save_username) == 0)
+						if(chatsender == m_save_username)
 							m_workspace->widgetChat()->receive(chatsender, chatmessage, KGGZChat::RECEIVE_OWN);
 						else
 						{
-							KGGZDEBUG("%s != %s\n", chatsender, m_save_username);
+							KGGZDEBUG("%s != %s\n", chatsender.utf8().data(),
+								m_save_username.utf8().data());
 							m_workspace->widgetChat()->receive(chatsender, chatmessage, KGGZChat::RECEIVE_CHAT);
 						}
 					}
@@ -780,7 +822,8 @@ void KGGZ::roomCollector(unsigned int id, void* data)
 					break;
 				case GGZCoreRoom::chatprivate:
 					KGGZDEBUG("chatprivate\n");
-					m_workspace->widgetChat()->receive(chatsender, chatmessage, KGGZChat::RECEIVE_PERSONAL);
+					if((m_grubby) && (m_grubby->isVisible())) m_grubby->answer(chatmessage);
+					else m_workspace->widgetChat()->receive(chatsender, chatmessage, KGGZChat::RECEIVE_PERSONAL);
 					break;
 				case GGZCoreRoom::chatbeep:
 					KGGZDEBUG("chatbeep\n");
@@ -794,21 +837,41 @@ void KGGZ::roomCollector(unsigned int id, void* data)
 			break;
 		case GGZCoreRoom::enter:
 			KGGZDEBUG("enter\n");
-			buffer.append((char*)data);
-			buffer.append(i18n(" enters the room."));
-			m_workspace->widgetChat()->receive(NULL, buffer.latin1(), KGGZChat::RECEIVE_ADMIN);
-			m_workspace->widgetUsers()->add((char*)data);
-			m_workspace->widgetChat()->chatline()->addPlayer((char*)data);
-			if(m_grubby) m_grubby->addPlayer((char*)data);
+			event = (GGZRoomChangeEventData*)data;
+			if(event->from_room == -2) fromroom = "(unknown origin)";
+			else if(event->from_room == -1) fromroom = i18n("first room after login");
+			else fromroom = QString("from room %1").arg(kggzserver->room(event->from_room)->name());
+			buffer = QString("%1 enters the room (%2).").arg(
+				event->player_name).arg(
+				fromroom);
+			//buffer.append(event->player_name);
+			//buffer.append(i18n(" enters the room."));
+			m_workspace->widgetChat()->receive(NULL, buffer, KGGZChat::RECEIVE_ADMIN);
+			m_workspace->widgetUsers()->add(event->player_name);
+			m_workspace->widgetChat()->chatline()->addPlayer(event->player_name);
+			//if(m_grubby) m_grubby->addPlayer(event->player_name);
+			roomnumber = event->to_room;
+			kggzroom = kggzserver->room(roomnumber);
+			emit signalRoomChanged(kggzroom->name(), kggzroom->gametype()->protocolEngine(), roomnumber, kggzroom->countPlayers());
 			break;
 		case GGZCoreRoom::leave:
 			KGGZDEBUG("leave\n");
-			buffer.append((char*)data);
-			buffer.append(i18n(" has left the room."));
-			m_workspace->widgetChat()->receive(NULL, buffer.latin1(), KGGZChat::RECEIVE_ADMIN);
-			m_workspace->widgetUsers()->remove((char*)data);
+			event = (GGZRoomChangeEventData*)data;
+			if(event->to_room == -2) toroom = "(unknown destination)";
+			else if(event->to_room == -1) toroom = i18n("disconnect from server");
+			else toroom = QString("to room %1").arg(kggzserver->room(event->to_room)->name());
+			buffer = QString("%1 has left the room (%2).").arg(
+				event->player_name).arg(
+				toroom);
+			//buffer.append(event->player_name);
+			//buffer.append(i18n(" has left the room."));
+			m_workspace->widgetChat()->receive(NULL, buffer, KGGZChat::RECEIVE_ADMIN);
+			m_workspace->widgetUsers()->remove(event->player_name);
 			KGGZDEBUG("remove from chatline\n");
-			m_workspace->widgetChat()->chatline()->removePlayer((char*)data);
+			m_workspace->widgetChat()->chatline()->removePlayer(event->player_name);
+			roomnumber = event->from_room;
+			kggzroom = kggzserver->room(roomnumber);
+			emit signalRoomChanged(kggzroom->name(), kggzroom->gametype()->protocolEngine(), roomnumber, kggzroom->countPlayers());
 			KGGZDEBUG("Leave room event: ready\n");
 			break;
 		case GGZCoreRoom::tableupdate:
@@ -841,7 +904,18 @@ void KGGZ::roomCollector(unsigned int id, void* data)
 			break;
 		case GGZCoreRoom::tableleft:
 			KGGZDEBUG("tableleft\n");
-			eventLeaveTable(0);
+			delete m_sn_game;
+			m_sn_game = NULL;
+			m_gamefd = -1;
+			if(kggzgame)
+			{
+				// XXX: what about eventLeaveGame?
+				detachGameCallbacks();
+				delete kggzgame;
+				kggzgame = NULL;
+			}
+			eventLeaveTable(0); /* was 1 in GGZCoreGame::over */
+			m_workspace->widgetChat()->receive(NULL, i18n("Game over"), KGGZChat::RECEIVE_ADMIN);
 			m_workspace->widgetChat()->receive(NULL, i18n("Left table"), KGGZChat::RECEIVE_ADMIN);
 			emit signalMenu(MENUSIG_GAMEOVER);
 			break;
@@ -856,17 +930,36 @@ void KGGZ::roomCollector(unsigned int id, void* data)
 		case GGZCoreRoom::playerlag:
 			lagPlayers();
 			break;
+		case GGZCoreRoom::stats:
+			m_workspace->widgetChat()->receive(NULL, i18n("Info: Statistics for %1 changed.").arg(player), KGGZChat::RECEIVE_ADMIN);
+			break;
+		case GGZCoreRoom::count:
+			//GGZCoreRoom *tmproom = new GGZCoreRoom();
+			KGGZDEBUG("count\n");
+			room = ggzcore_server_get_nth_room(kggzserver->server(), roomnumber);
+			emit signalRoomChanged(ggzcore_room_get_name(room),
+				ggzcore_gametype_get_prot_engine(ggzcore_room_get_gametype(room)),
+				roomnumber, ggzcore_room_get_num_players(room));
+			break;
 		default:
 			KGGZDEBUG("unknown\n");
 			break;
 	}
 }
 
-void KGGZ::serverCollector(unsigned int id, void* data)
+void KGGZ::serverCollector(unsigned int id, const void* data)
 {
 	int result;
 	QString buffer;
 	GGZCoreGametype *gametype;
+#ifdef KGGZ_WALLET
+	KWallet::Wallet *w = NULL;
+	QString entry;
+#endif
+	int roomnumber;
+	GGZMotdEventData *motd;
+
+	emit signalActivity(ACTIVITY_ROOM);
 
 	switch(id)
 	{
@@ -878,7 +971,8 @@ void KGGZ::serverCollector(unsigned int id, void* data)
 			if(!kggzserver) return;
 			m_sn_server = new QSocketNotifier(kggzserver->fd(), QSocketNotifier::Read, this);
 			connect(m_sn_server, SIGNAL(activated(int)), SLOT(slotServerData()));
-			kggzserver->setLogin(m_save_loginmode, m_save_username, m_save_password);
+			kggzserver->setLogin(m_save_loginmode,
+				m_save_username.utf8(), m_save_password.utf8());
 			break;
 		case GGZCoreServer::connectfail:
 			KGGZDEBUG("connectfail\n");
@@ -915,8 +1009,8 @@ void KGGZ::serverCollector(unsigned int id, void* data)
 		case GGZCoreServer::loggedin:
 			KGGZDEBUG("loggedin\n");
 			emit signalMenu(MENUSIG_LOGIN);
-			buffer.sprintf(i18n("logged in as %s@%s:%i (using TLS: %i)"), m_save_username, m_save_hostname, m_save_port, m_save_encryption);
-			emit signalCaption(buffer);
+			buffer = i18n("logged in as %1@%2:%3").arg(m_save_username).arg(m_save_hostname).arg(m_save_port);
+			emit signalCaption(buffer, m_save_encryption);
 			menuView(VIEW_CHAT);
 			if(m_save_loginmode == GGZCoreServer::firsttime)
 			{
@@ -924,8 +1018,23 @@ void KGGZ::serverCollector(unsigned int id, void* data)
 					i18n("You are welcome as a new GGZ Gaming Zone player.\n"
 						"Your personal password is: %1").arg(kggzserver->password()),
 					i18n("Information"));
+#ifdef KGGZ_WALLET
+				w = KWallet::Wallet::openWallet("kggz");
+				if(w)
+				{
+					entry = QString("%1@%2").arg(m_save_username).arg(m_save_hostname);
+					if(!w->hasFolder("passwords"))
+						w->createFolder("passwords");
+					w->setFolder("passwords");
+					w->writePassword(entry, kggzserver->password());
+				}
+				else KMessageBox::error(this,
+					i18n("The wallet could not be opened to insert the password (%1).").arg(
+						kggzserver->password()),
+					i18n("Connection"));
+#endif
 			}
-			buffer.sprintf(i18n("Logged in as %s"), m_save_username);
+			buffer = i18n("Logged in as %1").arg(m_save_username);
 			m_workspace->widgetChat()->receive(NULL, buffer, KGGZChat::RECEIVE_INFO);
 			m_workspace->widgetChat()->receive(NULL, i18n("Please join a room to start!"), KGGZChat::RECEIVE_INFO);
 			if((m_save_loginmode == GGZCoreServer::firsttime) && (m_motd)) m_motd->raise();
@@ -946,18 +1055,27 @@ void KGGZ::serverCollector(unsigned int id, void* data)
 		case GGZCoreServer::motdloaded:
 			KGGZDEBUG("motdloaded\n");
 			if(!m_motd) m_motd = new KGGZMotd(NULL, "KGGZMotd");
-			m_motd->setSource(data);
+			motd = (GGZMotdEventData*)data;
+			m_motd->setSource(motd->motd);
+			m_motd->setWebpage(motd->url);
 			if(m_showmotd) m_motd->show();
 			break;
 		case GGZCoreServer::roomlist:
 			KGGZDEBUG("roomlist\n");
-			emit signalMenu(MENUSIG_ROOMLIST);
-			for(int i = 0; i < kggzserver->countRooms(); i++)
-				emit signalRoom(kggzserver->room(i)->name(), kggzserver->room(i)->category());
 			break;
 		case GGZCoreServer::typelist:
 			KGGZDEBUG("typelist\n");
 
+			emit signalMenu(MENUSIG_ROOMLIST);
+			for(int i = 0; i < kggzserver->countRooms(); i++)
+			{
+				emit signalRoom(kggzserver->room(i)->name(),
+					kggzserver->room(i)->gametype()->protocolEngine(),
+					kggzserver->room(i)->category(),
+					kggzserver->room(i)->countPlayers());
+				m_roommap[kggzserver->room(i)->gametype()->protocolEngine()] = i;
+			}
+
 			if(!kggzroom)
 			{
 				menuView(VIEW_CHAT);
@@ -978,12 +1096,20 @@ void KGGZ::serverCollector(unsigned int id, void* data)
 			if(kggzroom->description())
 				m_workspace->widgetChat()->receive(NULL, i18n("(Description: %1)").arg(kggzroom->description()), KGGZChat::RECEIVE_ADMIN);
 			m_workspace->widgetUsers()->setRoom(kggzroom);
-			emit signalLocation(i18n("  Room: ") + kggzroom->name() + "  ");
+			emit signalLocation(i18n("Room: ") + kggzroom->name());
 			emit signalMenu(MENUSIG_ROOMENTER);
+
 			gametype = kggzroom->gametype();
 			KGGZDEBUG("Spectators allowed here? %i\n", gametype->maxSpectators());
 			if(gametype->maxSpectators()) emit signalMenu(MENUSIG_SPECTATORS);
 			else emit signalMenu(MENUSIG_NOSPECTATORS);
+
+			if(gamerulesurl()) emit signalMenu(MENUSIG_RULES);
+			else emit signalMenu(MENUSIG_NORULES);
+
+			roomnumber = m_roommap[kggzroom->gametype()->protocolEngine()];
+			emit signalRoomChanged(kggzroom->name(), kggzroom->gametype()->protocolEngine(),
+				roomnumber, kggzroom->countPlayers() + 1); // FIXME: bug in ggzcore?
 			break;
 		case GGZCoreServer::enterfail:
 			KGGZDEBUG("enterfail\n");
@@ -1041,7 +1167,19 @@ void KGGZ::serverCollector(unsigned int id, void* data)
 			m_channelfd = -1;
 
 			//game_channel_ready();
-			kggzgame->setFd(kggzserver->channel());
+			if(kggzgame)
+			{
+				kggzgame->setFd(kggzserver->channel());
+			}
+			else
+			{
+				// in case of table leave due to problem, kggzgame might be NULL
+				delete m_launch;
+				m_launch = NULL;
+				KMessageBox::error(this,
+					i18n("Could not establish channel due to failed launch!"),
+					i18n("Table launch error!"));
+			}
 
 			break;
 		case GGZCoreServer::channelfail:
@@ -1052,13 +1190,16 @@ void KGGZ::serverCollector(unsigned int id, void* data)
 				m_table = NULL;
 			}
 			break;
+		case GGZCoreServer::players:
+			emit signalPlayers(kggzserver->countPlayers());
+			break;
 		default:
 			KGGZDEBUG("unknown\n");
 			break;
 	}
 }
 
-GGZHookReturn KGGZ::hookOpenCollector(unsigned int id, void* event_data, void* user_data)
+GGZHookReturn KGGZ::hookOpenCollector(unsigned int id, const void* event_data, const void* user_data)
 {
 	KGGZCallback* kggzcallback;
 	KGGZ* kggz;
@@ -1096,24 +1237,21 @@ GGZHookReturn KGGZ::hookOpenCollector(unsigned int id, void* event_data, void* u
 	return GGZ_HOOK_OK;
 }
 
-void KGGZ::slotChat(const char *text, char *player, int mode)
+void KGGZ::slotChat(QString text, QString player, int mode)
 {
-	char *sendtext;
-
 	if((kggzserver) && (kggzroom))
 	{
-		KGGZDEBUG("Chat sends: %s\n", text);
-		sendtext = strdup(text);
+		KGGZDEBUG("Chat sends: %s\n", text.utf8().data());
 		switch(mode)
 		{
 			case KGGZChat::RECEIVE_CHAT:
-				kggzroom->chat(GGZ_CHAT_NORMAL, player, sendtext);
+				kggzroom->chat(GGZ_CHAT_NORMAL, player, text);
 				break;
 			case KGGZChat::RECEIVE_TABLE:
-				kggzroom->chat(GGZ_CHAT_TABLE, player, sendtext);
+				kggzroom->chat(GGZ_CHAT_TABLE, player, text);
 				break;
 			case KGGZChat::RECEIVE_ANNOUNCE:
-				if(kggzroom->chat(GGZ_CHAT_ANNOUNCE, NULL, sendtext) < 0)
+				if(kggzroom->chat(GGZ_CHAT_ANNOUNCE, NULL, text) < 0)
 				{
 					m_workspace->widgetChat()->receive(NULL,
 						i18n("Only administrators are allowed to broadcast messages."),
@@ -1121,10 +1259,10 @@ void KGGZ::slotChat(const char *text, char *player, int mode)
 				}
 				break;
 			case KGGZChat::RECEIVE_BEEP:
-				kggzroom->chat(GGZ_CHAT_BEEP, player, sendtext);
+				kggzroom->chat(GGZ_CHAT_BEEP, player, text);
 				break;
 			case KGGZChat::RECEIVE_PERSONAL:
-				kggzroom->chat(GGZ_CHAT_PERSONAL, player, sendtext);
+				kggzroom->chat(GGZ_CHAT_PERSONAL, player, text);
 				break;
 		}
 	}
@@ -1148,7 +1286,8 @@ void KGGZ::attachRoomCallbacks()
 	kggzroom->addHook(GGZCoreRoom::tableleft, &KGGZ::hookOpenCollector, (void*)kggzroomcallback);
 	kggzroom->addHook(GGZCoreRoom::tableleavefail, &KGGZ::hookOpenCollector, (void*)kggzroomcallback);
 	/*kggzroom->addHook(GGZCoreRoom::tabledata, &KGGZ::hookOpenCollector, (void*)kggzroomcallback);*/
-	kggzroom->addHook(GGZCoreRoom::playerlag, &KGGZ::hookOpenCollector, (void*)kggzroomcallback);
+	kggzroom->addHook(GGZCoreRoom::stats, &KGGZ::hookOpenCollector, (void*)kggzroomcallback);
+	kggzroom->addHook(GGZCoreRoom::count, &KGGZ::hookOpenCollector, (void*)kggzroomcallback);
 }
 
 void KGGZ::detachRoomCallbacks()
@@ -1168,6 +1307,8 @@ void KGGZ::detachRoomCallbacks()
 	kggzroom->removeHook(GGZCoreRoom::tableleavefail, &KGGZ::hookOpenCollector);
 	/*kggzroom->removeHook(GGZCoreRoom::tabledata, &KGGZ::hookOpenCollector);*/
 	kggzroom->removeHook(GGZCoreRoom::playerlag, &KGGZ::hookOpenCollector);
+	kggzroom->removeHook(GGZCoreRoom::stats, &KGGZ::hookOpenCollector);
+	kggzroom->removeHook(GGZCoreRoom::count, &KGGZ::hookOpenCollector);
 }
 
 void KGGZ::attachServerCallbacks()
@@ -1192,6 +1333,7 @@ void KGGZ::attachServerCallbacks()
 	kggzserver->addHook(GGZCoreServer::channelconnected, &KGGZ::hookOpenCollector, (void*)kggzservercallback);
 	kggzserver->addHook(GGZCoreServer::channelready, &KGGZ::hookOpenCollector, (void*)kggzservercallback);
 	kggzserver->addHook(GGZCoreServer::channelfail, &KGGZ::hookOpenCollector, (void*)kggzservercallback);
+	kggzserver->addHook(GGZCoreServer::players, &KGGZ::hookOpenCollector, (void*)kggzservercallback);
 }
 
 void KGGZ::detachServerCallbacks()
@@ -1225,9 +1367,6 @@ void KGGZ::attachGameCallbacks()
 	kggzgame->addHook(GGZCoreGame::launchfail, &KGGZ::hookOpenCollector, (void*)kggzgamecallback);
 	kggzgame->addHook(GGZCoreGame::negotiated, &KGGZ::hookOpenCollector, (void*)kggzgamecallback);
 	kggzgame->addHook(GGZCoreGame::negotiatefail, &KGGZ::hookOpenCollector, (void*)kggzgamecallback);
-	kggzgame->addHook(GGZCoreGame::over, &KGGZ::hookOpenCollector, (void*)kggzgamecallback);
-	kggzgame->addHook(GGZCoreGame::ioerror, &KGGZ::hookOpenCollector, (void*)kggzgamecallback);
-	kggzgame->addHook(GGZCoreGame::protoerror, &KGGZ::hookOpenCollector, (void*)kggzgamecallback);
 	kggzgame->addHook(GGZCoreGame::playing, &KGGZ::hookOpenCollector, (void*)kggzgamecallback);
 }
 
@@ -1238,9 +1377,6 @@ void KGGZ::detachGameCallbacks()
 	kggzgame->removeHook(GGZCoreGame::launchfail, &KGGZ::hookOpenCollector);
 	kggzgame->removeHook(GGZCoreGame::negotiated, &KGGZ::hookOpenCollector);
 	kggzgame->removeHook(GGZCoreGame::negotiatefail, &KGGZ::hookOpenCollector);
-	kggzgame->removeHook(GGZCoreGame::over, &KGGZ::hookOpenCollector);
-	kggzgame->removeHook(GGZCoreGame::ioerror, &KGGZ::hookOpenCollector);
-	kggzgame->removeHook(GGZCoreGame::protoerror, &KGGZ::hookOpenCollector);
 	kggzgame->removeHook(GGZCoreGame::playing, &KGGZ::hookOpenCollector);
 }
 
@@ -1301,9 +1437,6 @@ void KGGZ::menuView(int viewtype)
 	// Qt is crappy here... so we must do something for the chicks (animation)
 	m_workspace->resize(width() - 1, height() - 1);
 	m_workspace->resize(width(), height());
-
-	// Don't hide connection dialog
-	if(m_connect) m_connect->raise();
 }
 
 void KGGZ::menuGameLaunch()
@@ -1385,6 +1518,8 @@ void KGGZ::slotGameFrontend()
 {
 	GGZCoreGametype *gametype;
 	int modules;
+	int env;
+	int ret;
 
 	if((!kggzroom) || (!m_gameinfo))
 	{
@@ -1394,6 +1529,8 @@ void KGGZ::slotGameFrontend()
 
 	gametype = kggzroom->gametype();
 
+	m_core->reload();
+
 	KGGZDEBUG("Create module...\n");
 	m_module = new GGZCoreModule();
 	m_module->init(gametype->name(), gametype->protocolVersion(), gametype->protocolEngine());
@@ -1416,6 +1553,20 @@ void KGGZ::slotGameFrontend()
 		return;
 	}
 
+	if(gametype->allowPeers())
+	{
+		ret = KMessageBox::questionYesNo(this,
+			i18n("The selected game might reveal your IP address or hostname "
+				"to other players. Do you still want to continue?"),
+			i18n("Player Privacy"));
+		if(ret != KMessageBox::Yes)
+		{
+			delete m_module;
+			m_module = NULL;
+			return;
+		}
+	}
+
 	if(modules == 1) slotGamePrepare(0);
 	else
 	{
@@ -1424,7 +1575,9 @@ void KGGZ::slotGameFrontend()
 		for(int i = 0; i < modules; i++)
 		{
 			m_module->setActive(i);
-			m_selector->addFrontend(m_module->frontend());
+			env = m_module->environment();
+			if((env == GGZCoreModule::xwindow) || (env == GGZCoreModule::xfullscreen))
+				m_selector->addFrontend(m_module->game(), m_module->frontend(), i);
 		}
 		m_selector->show();
 		connect(m_selector, SIGNAL(signalFrontend(int)), SLOT(slotGamePrepare(int)));
@@ -1455,6 +1608,35 @@ void KGGZ::slotGamePrepare(int frontend)
 			KGGZDEBUG("Assignment: %i is %i\n", i, gametype->isPlayersValid(i + 1));
 			m_launch->setSeatAssignment(i, gametype->isPlayersValid(i + 1));
 		}
+		KGGZDEBUG("*** %i named bots\n", gametype->namedBots());
+		for(int i = 0; i < gametype->namedBots(); i++)
+		{
+			m_launch->addBot(gametype->namedBotName(i), gametype->namedBotClass(i));
+		}
+
+		KSimpleConfig conf(QDir::home().path() + "/.ggz/kggz.rc");
+		QMap<QString, QString> credits = conf.entryMap("Credits");
+		QMap<QString, QString>::Iterator it;
+		for(it = credits.begin(); it != credits.end(); it++)
+		{
+			int credit = it.data().toInt();
+			if(credit == 1) // KGGZUsers::creditbuddy
+			{
+				m_launch->addBuddy(it.key());
+			}
+		}
+
+		int number = kggzroom->countPlayers();
+		for(int i = 0; i < number; i++)
+		{
+			GGZCorePlayer *player = kggzroom->player(i);
+			QString playername = player->name();
+			if(player->type() == KGGZUsers::assignbot)
+			{
+				m_launch->addBot(playername, "grubby");
+			}
+		}
+
 		m_launch->show();
 		connect(m_launch, SIGNAL(signalLaunch()), SLOT(slotGameStart()));
 	}
@@ -1558,6 +1740,9 @@ void KGGZ::menuGameInfo()
 		module->setActive(i);
 		buffer.append(QString("<b>%1</b>").arg(i18n("Game client")));
 		buffer.append("<br>");
+		buffer.append(i18n("Name: "));
+		buffer.append(module->game());
+		buffer.append("<br>");
 		buffer.append(i18n("Frontend: "));
 		buffer.append(module->frontend());
 		buffer.append("<br>");
@@ -1576,6 +1761,36 @@ void KGGZ::menuGameInfo()
 	m_gameinfodialog->show();
 }
 
+QString KGGZ::gamerulesurl()
+{
+	GGZCoreGametype *gametype = kggzroom->gametype();
+	kapp->config()->setGroup("Rules");
+	QString rulesurl = kapp->config()->readEntry(gametype->name());
+	return rulesurl;
+}
+
+void KGGZ::menuGameRules()
+{
+	GGZCoreGametype *gametype;
+	if(!kggzserver)
+		return;
+	if(!kggzroom)
+	{
+		KMessageBox::sorry(this, i18n("Please join a room first."), i18n("Game information failure"));
+		return;
+	}
+	gametype = kggzroom->gametype();
+	if(!gametype)
+		return;
+
+#ifdef KGGZ_BROWSER
+	m_browser->request(gamerulesurl());
+	menuView(VIEW_BROWSER);
+#else
+	kapp->invokeBrowser(gamerulesurl());
+#endif
+}
+
 void KGGZ::menuRoom(int room)
 {
 	if(room >= 0)
@@ -1605,23 +1820,25 @@ void KGGZ::slotLoadLogo()
 	gametype = kggzroom->gametype();
 	if((!gametype) || (!gametype->name()))
 	{
-		m_workspace->widgetLogo()->setLogo(NULL, "lounge");
+		m_workspace->widgetLogo()->setLogo(NULL, "lounge", true);
 		return;
 	}
 	KGGZDEBUG("__ more loading: %s %s %s\n", gametype->name(), gametype->protocolVersion(), gametype->protocolEngine());
 
+	m_core->reload();
+
 	module = new GGZCoreModule();
 	module->init(gametype->name(), gametype->protocolVersion(), gametype->protocolEngine());
 	if(module->count() == 0)
 	{
-		m_workspace->widgetLogo()->setLogo(NULL, "notinstalled");
+		m_workspace->widgetLogo()->setLogo(NULL, gametype->protocolEngine(), false);
 		delete module;
 		return;
 	}
 	module->setActive(0);
 
 	icon = module->pathIcon();
-	m_workspace->widgetLogo()->setLogo(icon, gametype->protocolEngine());
+	m_workspace->widgetLogo()->setLogo(icon, gametype->protocolEngine(), true);
 
 	delete module;
 }
@@ -1638,36 +1855,36 @@ void KGGZ::menuGrubby()
 	m_grubby->show();
 	listPlayers();
 	listTables();
-	connect(m_grubby, SIGNAL(signalAction(const char*, const char*, int)), SLOT(slotGrubby(const char*, const char*, int)));
+	connect(m_grubby, SIGNAL(signalAction(QString, QString, int)), SLOT(slotGrubby(QString, QString, int)));
 }
 
-void KGGZ::slotGrubby(const char *grubby, const char *argument, int id)
+void KGGZ::slotGrubby(QString grubby, QString argument, int id)
 {
 	switch(id)
 	{
 		case KGGZGrubby::actionabout:
-			slotChat(QString("%1 about").arg(grubby).latin1(), NULL, KGGZChat::RECEIVE_CHAT);
+			slotChat("about", grubby, KGGZChat::RECEIVE_PERSONAL);
 			break;
 		case KGGZGrubby::actionhelp:
-			slotChat(QString("%1 help").arg(grubby).latin1(), NULL, KGGZChat::RECEIVE_CHAT);
+			slotChat("help", grubby, KGGZChat::RECEIVE_PERSONAL);
 			break;
 		case KGGZGrubby::actionseen:
-			slotChat(QString("%1 have you seen %2").arg(grubby).arg(argument).latin1(), NULL, KGGZChat::RECEIVE_CHAT);
+			slotChat(QString("have you seen %1").arg(argument), grubby, KGGZChat::RECEIVE_PERSONAL);
 			break;
 		case KGGZGrubby::actionalertadd:
-			slotChat(QString("%1 alert %2").arg(grubby).arg(argument).latin1(), NULL, KGGZChat::RECEIVE_CHAT);
+			slotChat(QString("alert %1").arg(argument), grubby, KGGZChat::RECEIVE_PERSONAL);
 			break;
 		case KGGZGrubby::actionmessages:
-			slotChat(QString("%1 do i have any messages").arg(grubby).latin1(), NULL, KGGZChat::RECEIVE_CHAT);
+			slotChat("do i have any messages", grubby, KGGZChat::RECEIVE_PERSONAL);
 			break;
 		case KGGZGrubby::actionbye:
 			KMessageBox::information(m_grubby, i18n("The pleasure has been on my side :)"), "Grubby");
 			break;
 		case KGGZGrubby::actionteach:
-			slotChat(QString("%1 %2").arg(grubby).arg(argument), NULL, KGGZChat::RECEIVE_CHAT);
+			slotChat(QString("%1").arg(argument), grubby, KGGZChat::RECEIVE_PERSONAL);
 			break;
 		case KGGZGrubby::actionwhois:
-			slotChat(QString("%1 who is %2").arg(grubby).arg(argument), NULL, KGGZChat::RECEIVE_CHAT);
+			slotChat(QString("who is %1").arg(argument), grubby, KGGZChat::RECEIVE_PERSONAL);
 			break;
 		default:
 			KMessageBox::sorry(m_grubby, i18n("I don't know that command, sorry."), i18n("Grubby communication failure"));
@@ -1712,13 +1929,6 @@ void KGGZ::eventLeaveTable(int force)
 		if(m_gameinfo->type() == KGGZGameInfo::typespectator) kggzroom->leaveTableSpectator();
 		else kggzroom->leaveTable(force);
 	}
-
-	//if((kggzserver) && (force))
-	//{
-	//	listPlayers();
-	//	listTables();
-	//}
-	//emit signalMenu(MENUSIG_GAMEOVER);
 }
 
 void KGGZ::eventLeaveRoom()
@@ -1737,7 +1947,7 @@ void KGGZ::eventLeaveRoom()
 	m_workspace->widgetChat()->shutdown();
 	m_workspace->widgetLogo()->shutdown();
 	m_workspace->widgetUsers()->setRoom(NULL);
-	emit signalLocation(i18n("  No room selected  "));
+	emit signalLocation(i18n("No room selected"));
 }
 
 void KGGZ::menuGameCancel()
@@ -1752,3 +1962,10 @@ void KGGZ::menuGameTeam()
 	m_team->show();
 }
 
+void KGGZ::showEvent(QShowEvent *e)
+{
+	Q_UNUSED(e);
+
+	emit signalActivity(ACTIVITY_NONE);
+}
+
