@@ -42,6 +42,8 @@
 // KDE classes
 #include <kmessagebox.h>
 #include <klocale.h>
+#include <kapplication.h>
+#include <kurl.h>
 
 // Qt classes
 #include <qlayout.h>
@@ -54,23 +56,33 @@
 #include <qcheckbox.h>
 #include <qbuttongroup.h>
 #include <qsocket.h>
+#include <qeventloop.h>
+#include <qdir.h>
 
 // System includes
 #include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
+
+#ifdef WITH_HOWL
+#include <sys/types.h>
+#include <howl.h>
+#endif
 
 // GGZCore++ includes
 #include "GGZCoreConfio.h"
 
+#ifdef WITH_HOWL
+static KGGZConnect *connectobj = NULL;
+#endif
+static QString connectstr = QString::null;
+
 /* Constructor: set up a small dialog for connections; provide server profile list */
 KGGZConnect::KGGZConnect(QWidget *parent, const char *name)
-: QWidget(parent, name, WStyle_Customize | WStyle_Tool | WStyle_DialogBorder)
+: QWidget(parent, name, WType_Dialog)
 {
 	QVBoxLayout *vbox1, *vbox2;
 	QHBoxLayout *hbox1, *hbox2, *hbox3, *hbox4;
 	QLabel *label_host, *label_port, *label_name, *label_password;
-	QRadioButton *mode_normal, *mode_guest, *mode_firsttime;
+	QRadioButton *radio_normal, *radio_guest, *radio_firsttime;
 	QLabel *label_mode;
 	QIntValidator *valid;
 	KGGZLineSeparator *sep1, *sep2;
@@ -111,7 +123,12 @@ KGGZConnect::KGGZConnect(QWidget *parent, const char *name)
 	input_port->setValidator(valid);
 	input_name = new QLineEdit(m_pane);
 	input_password = new QLineEdit(m_pane);
+
+#ifdef KGGZ_WALLET
+	input_password->setEnabled(false);
+#else
 	input_password->setEchoMode(QLineEdit::Password);
+#endif
 
 	button_ok = new QPushButton(i18n("Connect"), this);
 	button_cancel = new QPushButton(i18n("Cancel"), this);
@@ -119,13 +136,13 @@ KGGZConnect::KGGZConnect(QWidget *parent, const char *name)
 	label_mode = new QLabel(i18n("Log in as:"), m_pane);
 	group_mode = new QButtonGroup(m_pane);
 	group_mode->hide();
-	mode_normal = new QRadioButton(i18n("normal"), m_pane);
-	mode_guest = new QRadioButton(i18n("guest"), m_pane);
-	mode_firsttime = new QRadioButton(i18n("starter"), m_pane);
-	group_mode->insert(mode_normal);
-	group_mode->insert(mode_guest);
-	group_mode->insert(mode_firsttime);
-	mode_normal->setChecked(true);
+	radio_normal = new QRadioButton(i18n("normal"), m_pane);
+	radio_guest = new QRadioButton(i18n("guest"), m_pane);
+	radio_firsttime = new QRadioButton(i18n("starter"), m_pane);
+	group_mode->insert(radio_normal, mode_normal);
+	group_mode->insert(radio_guest, mode_guest);
+	group_mode->insert(radio_firsttime, mode_firsttime);
+	radio_normal->setChecked(true);
 
 	option_server = new QCheckBox(i18n("Start ggzd server locally"), this);
 
@@ -165,9 +182,9 @@ KGGZConnect::KGGZConnect(QWidget *parent, const char *name)
 
 	hbox3 = new QHBoxLayout(vbox2, 5);
 	hbox3->add(label_mode);
-	hbox3->add(mode_normal);
-	hbox3->add(mode_guest);
-	hbox3->add(mode_firsttime);
+	hbox3->add(radio_normal);
+	hbox3->add(radio_guest);
+	hbox3->add(radio_firsttime);
 
 	vbox2->add(sep2);
 
@@ -193,11 +210,12 @@ KGGZConnect::KGGZConnect(QWidget *parent, const char *name)
 	connect(button_select, SIGNAL(clicked()), SLOT(slotProfileMeta()));
 
 	m_connected = -1;
-	m_loginmode = 0;
+	m_loginmode = mode_normal;
 	m_nosafe = 0;
 
 	setCaption(i18n("Connect to server"));
 
+	slotModes(m_loginmode);
 	slotLoadProfile(-1);
 }
 
@@ -209,17 +227,23 @@ KGGZConnect::~KGGZConnect()
 void KGGZConnect::slotSaveProfile()
 {
 	GGZCoreConfio *config;
-	const char *current;
+	QCString current;
 
-	config = new GGZCoreConfio(QString("%1/.ggz/kggz.rc").arg(getenv("HOME")).latin1(), GGZCoreConfio::readwrite | GGZCoreConfio::create);
+	config = new GGZCoreConfio(QString("%1/.ggz/kggz.rc").arg(QDir::homeDirPath()).utf8(),
+		GGZCoreConfio::readwrite | GGZCoreConfio::create);
 
 	// Save previous profile and make new one the default (although it's empty)
-	current = m_current.latin1();
+	current = m_current.utf8();
 	config->write(current, "Host", input_host->text());
 	config->write(current, "Port", input_port->text());
 	config->write(current, "Login", input_name->text());
+#ifndef KGGZ_WALLET
 	config->write(current, "Password", input_password->text());
+#endif
 	config->write(current, "Type", m_loginmode);
+#ifdef KGGZ_PATCH_ENCRYPTION
+	config->write(current, "Encryption", option_tls->isChecked());
+#endif
 	config->write("Session", "Defaultserver", profile_select->currentText());
 	config->commit();
 	delete config;
@@ -231,43 +255,53 @@ void KGGZConnect::slotLoadProfile(int profile)
 	GGZCoreConfio *config;
 	char **list = NULL;
 	int i;
-	char *host = NULL;
-	char *port = NULL;
-	char *username = NULL;
-	char *password = NULL;
 	int type;
 	const char *listentry = NULL;
+	bool usetls;
+	QString server, defaultserver;
+	QString host, port, username, password;
 
 	if((profile != -1) && (!m_nosafe))
 		slotSaveProfile();
 	m_nosafe = 0;
 
-	config = new GGZCoreConfio(QString("%1/.ggz/kggz.rc").arg(getenv("HOME")).latin1(), GGZCoreConfio::readonly);
+	config = new GGZCoreConfio(QString("%1/.ggz/kggz.rc").arg(QDir::homeDirPath()).utf8(),
+		GGZCoreConfio::readonly);
 
 	// setup entries; try to find suitable first entry
 	if(profile == -1)
 	{
 		profile_select->clear();
 		listentry = config->read("Session", "Defaultserver", (char*)NULL);
-		if(listentry)
+		defaultserver = listentry;
+		if(!defaultserver.isNull())
 		{
-			if(strcmp(listentry, i18n("GGZ Meta Server").latin1()))
-				profile_select->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/server.png"), listentry);
-			profile_select->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/metaserver.png"), i18n("GGZ Meta Server"));
-			profile_select->setCurrentItem(0);
+			if((defaultserver != i18n("GGZ Meta Server"))
+			&& ((defaultserver != i18n("LAN Game"))))
+			{
+				profile_select->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/server.png"), defaultserver);
+				profile_select->setCurrentItem(0);
+			}
 		}
-		else
-		{
-			profile_select->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/metaserver.png"), i18n("GGZ Meta Server"));
+		profile_select->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/metaserver.png"), i18n("GGZ Meta Server"));
+		if(defaultserver == i18n("GGZ Meta Server"))
 			profile_select->setCurrentItem(0);
-			modifyServerList(i18n("Default CVS Developer Server"), 1);
+#ifdef WITH_HOWL
+		profile_select->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/langame.png"), i18n("LAN Game"));
+		if(defaultserver == i18n("LAN Game"))
+			profile_select->setCurrentItem(1);
+#endif
+		if(defaultserver.isNull())
+		{
+			modifyServerList(i18n("Default Stable Server"), 1);
 		}
 		
 		config->read("Servers", "Servers", &i, &list);
 		for(int j = 0; j < i; j++)
 		{
-			if((listentry) && (strcmp(list[j], listentry)))
-				profile_select->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/server.png"), list[j]);
+			server = list[j];
+			if(server != defaultserver)
+				profile_select->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/server.png"), server);
 			GGZCoreConfio::free(list[j]);
 		}
 		if(list) GGZCoreConfio::free(list);
@@ -277,8 +311,17 @@ void KGGZConnect::slotLoadProfile(int profile)
 	// read values for selected server
 	if(profile_select->currentText() == i18n("GGZ Meta Server"))
 	{
-		host = strdup(i18n("Automatic"));
-		port = strdup("5688");
+		host = i18n("Automatic");
+		port = "5688";
+		input_host->setEnabled(false);
+		input_port->setEnabled(false);
+		button_select->setEnabled(false);
+		profile_delete->setEnabled(false);
+	}
+	else if(profile_select->currentText() == i18n("LAN Game"))
+	{
+		host = i18n("Autodetect");
+		port = "5688";
 		input_host->setEnabled(false);
 		input_port->setEnabled(false);
 		button_select->setEnabled(false);
@@ -286,7 +329,7 @@ void KGGZConnect::slotLoadProfile(int profile)
 	}
 	else
 	{
-		host = config->read(listentry, "Host", "jzaun.com");
+		host = config->read(listentry, "Host", "live.ggzgamingzone.org");
 		port = config->read(listentry, "Port", "5688");
 		input_host->setEnabled(true);
 		input_port->setEnabled(true);
@@ -296,20 +339,20 @@ void KGGZConnect::slotLoadProfile(int profile)
 	username = config->read(listentry, "Login", i18n("GGZocker"));
 	password = config->read(listentry, "Password", "");
 	type = config->read(listentry, "Type", 1);
+	usetls = config->read(listentry, "Encryption", false);
 
 	// put values into fields
 	input_host->setText(host);
 	input_port->setText(port);
 	input_name->setText(username);
+#ifndef KGGZ_WALLET
 	input_password->setText(password);
+#endif
+#ifdef KGGZ_PATCH_ENCRYPTION
+	option_tls->setChecked(usetls);
+#endif
 	slotModes(type);
 
-	// free allocated memory
-	if(host) GGZCoreConfio::free(host);
-	if(port) GGZCoreConfio::free(port);
-	if(username) GGZCoreConfio::free(username);
-	if(password) GGZCoreConfio::free(password);
-
 	// get rid of the configuration object
 	delete config;
 
@@ -326,18 +369,22 @@ void KGGZConnect::slotAccept()
 
 	button_ok->setEnabled(false);
 
-	if(input_host->text() != i18n("Automatic"))
-	{
-		close();
-		emit signalConnect(input_host->text().latin1(), atoi(input_port->text().latin1()),
-			input_name->text().latin1(), input_password->text().latin1(), m_loginmode);
-	}
-	else
+	if(input_host->text() == i18n("Automatic"))
 	{
 		m_sock = new QSocket();
 		connect(m_sock, SIGNAL(connected()), SLOT(slotWrite()));
 		connect(m_sock, SIGNAL(readyRead()), SLOT(slotRead()));
-		m_sock->connectToHost("ggz.snafu.de", 15689);
+		m_sock->connectToHost("live.ggzgamingzone.org", 15689);
+	}
+	else if(input_host->text() == i18n("Autodetect"))
+	{
+		zeroconfQuery();
+	}
+	else
+	{
+		close();
+		emit signalConnect(input_host->text(), input_port->text().toInt(),
+			input_name->text(), input_password->text(), m_loginmode);
 	}
 }
 
@@ -345,10 +392,23 @@ void KGGZConnect::slotModes(int loginmode)
 {
 	m_loginmode = loginmode;
 	group_mode->setButton(loginmode);
-	if(loginmode == 0)
+
+	if(loginmode == mode_normal)
+	{
+#ifdef KGGZ_WALLET
+		input_password->setText(i18n("Using KWallet"));
+#else
 		input_password->setEnabled(true);
+#endif
+	}
 	else
+	{
+#ifdef KGGZ_WALLET
+		input_password->setText("");
+#else
 		input_password->setEnabled(false);
+#endif
+	}
 }
 
 void KGGZConnect::slotInvoke()
@@ -369,10 +429,11 @@ void KGGZConnect::slotProfileNew()
 	}
 
 	// show input box and connect signal to slot
-	if(!m_input) m_input = new KGGZInput(NULL, "KGGZInput", i18n("Profile identifier"), i18n("Chose a name for the new profile."));
+	if(!m_input) m_input = new KGGZInput(NULL, "KGGZInput",
+		i18n("Profile identifier"), i18n("Chose a name for the new profile."));
 
 	m_input->show();
-	connect(m_input, SIGNAL(signalText(const char*)), SLOT(slotProfileProcess(const char*)));
+	connect(m_input, SIGNAL(signalText(QString)), SLOT(slotProfileProcess(QString)));
 }
 
 void KGGZConnect::slotProfileMeta()
@@ -394,14 +455,16 @@ void KGGZConnect::slotProfileMetaProcess(QString host, QString port)
 
 void KGGZConnect::slotProfileDelete()
 {
-	modifyServerList(m_current.latin1(), 2);
+	modifyServerList(m_current, 2);
 	profile_select->removeItem(profile_select->currentItem());
 	m_nosafe = 1;
 	slotLoadProfile(profile_select->currentItem());
 }
 
-void KGGZConnect::slotProfileProcess(const char *identifier)
+void KGGZConnect::slotProfileProcess(QString identifier)
 {
+	if(identifier.isEmpty()) return;
+
 	// Save current profile
 	slotSaveProfile();
 
@@ -409,7 +472,7 @@ void KGGZConnect::slotProfileProcess(const char *identifier)
 	profile_select->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/server.png"), identifier, 0);
 	profile_select->setCurrentItem(0);
 	modifyServerList(identifier, 1);
-	m_current = QString(identifier);
+	m_current = identifier;
 	input_host->setText("");
 	input_name->setText("");
 	input_port->setText("5688");
@@ -425,44 +488,35 @@ void KGGZConnect::slotProfileProcess(const char *identifier)
 }
 
 // mode: 1 = add, 2 = delete
-void KGGZConnect::modifyServerList(const char *server, int mode)
+void KGGZConnect::modifyServerList(QString server, int mode)
 {
 	GGZCoreConfio *config;
 	char **list = NULL;
-	char **list2 = NULL;
 	int i;
-	int number;
+	QStringList servers;
+	QString oldserver;
 
-	config = new GGZCoreConfio(QString("%1/.ggz/kggz.rc").arg(getenv("HOME")).latin1(), GGZCoreConfio::readwrite | GGZCoreConfio::create);
+	server = server.replace(" ", "\\ ");
+
+	config = new GGZCoreConfio(QString("%1/.ggz/kggz.rc").arg(QDir::homeDirPath()).utf8(),
+		GGZCoreConfio::readwrite | GGZCoreConfio::create);
 
 	// Update the list of available servers
 	config->read("Servers", "Servers", &i, &list);
 
-	list2 = (char**)malloc((i + 2) * sizeof(char*));
-	for(int j = 0; j < i + 2; j++)
-		list2[j] = NULL;
-
-	number = 0;
 	for(int j = 0; j < i; j++)
 	{
-		if(strcmp(list[j], server) || (mode == 1))
-		{
-			list2[number] = (char*)malloc(strlen(list[j]) + 1);
-			strcpy(list2[number], list[j]);
-			number++;
-		}
+		oldserver = QString(list[j]).replace(" ", "\\ ");
+		if((server != oldserver) || (mode == 1))
+			servers << oldserver;
 	}
-	if(mode == 1)
-	{
-		list2[number] = (char*)malloc(strlen(server) + 1);
-		strcpy(list2[number], server);
-		number++;
-	}
-	config->write("Servers", "Servers", number, list2);
+	if(mode == 1) servers << server;
+
+	config->write("Servers", "Servers", servers.join(" ").utf8());
 	if(mode == 2)
 	{
-		config->removeSection(server);
-		if(number == 0) config->removeKey("Servers", "Servers");
+		config->removeSection(server.utf8());
+		if(i < 2) config->removeKey("Servers", "Servers");
 	}
 
 	config->commit();
@@ -470,10 +524,7 @@ void KGGZConnect::modifyServerList(const char *server, int mode)
 
 	for(int j = 0; j < i; j++)
 		GGZCoreConfio::free(list[j]);
-	for(int j = 0; j < number; j++)
-		free(list2[j]);
 	if(list) GGZCoreConfio::free(list);
-	if(list2) free(list2);
 }
 
 void KGGZConnect::slotPane()
@@ -524,7 +575,7 @@ void KGGZConnect::slotRead()
 			host = host.right(host.length() - 2);
 			port = *(list.at(2));
 			close();
-			emit signalConnect(host, atoi(port.latin1()), input_name->text().latin1(), input_password->text().latin1(), m_loginmode);
+			emit signalConnect(host, port.toInt(), input_name->text(), input_password->text(), m_loginmode);
 		}
 	}
 }
@@ -534,12 +585,84 @@ void KGGZConnect::slotWrite()
 	QString s;
 
 	s = QString("query://ggz/connection/%1\n").arg(KGGZVERSION);
-	m_sock->writeBlock(s.latin1(), s.length());
+	m_sock->writeBlock(s.utf8(), s.length());
 	m_sock->flush();
 }
 
 void KGGZConnect::showEvent(QShowEvent *e)
 {
+	Q_UNUSED(e);
+
 	button_ok->setEnabled(true);
 }
 
+#ifdef WITH_HOWL
+static sw_result reply(sw_discovery session, sw_discovery_oid oid,
+	sw_uint32 interface,
+	sw_const_string name, sw_const_string type, sw_const_string domain,
+	sw_ipv4_address address, sw_port port, sw_octets tx, sw_ulong txl,
+	sw_opaque extra)
+{
+	sw_string str;
+
+	str = (char*)malloc(100);
+	str = sw_ipv4_address_name(address, str, 100);
+
+	connectstr = QString("ggz://%1:%2").arg(str).arg(port);
+	return SW_OKAY;
+}
+
+static sw_result breply(sw_discovery session, sw_discovery_oid oid,
+	sw_discovery_browse_status status, sw_uint32 interface,
+	sw_const_string name, sw_const_string type, sw_const_string domain,
+	sw_opaque extra)
+{
+	int ret;
+
+	if(status == SW_DISCOVERY_BROWSE_ADD_SERVICE)
+	{
+		ret = sw_discovery_resolve(session, interface, name, type, domain, reply, NULL, &oid);
+		if(ret)
+		{
+			KMessageBox::error(connectobj, i18n("Resolving failed."), i18n("Zeroconf error"));
+			return -1;
+		}
+	}
+
+	return SW_OKAY;
+}
+#endif
+
+void KGGZConnect::zeroconfQuery()
+{
+#ifdef WITH_HOWL
+	int ret;
+	sw_discovery session;
+	sw_discovery_oid oid;
+
+	connectobj = this;
+	connectstr = QString::null;
+
+	ret = sw_discovery_init(&session);
+	if(ret != SW_OKAY)
+	{
+		KMessageBox::error(this, i18n("Zeroconf could not be initialized."), i18n("Zeroconf error"));
+		return;
+	}
+
+	ret = sw_discovery_browse(session, 0, "_ggz._tcp", NULL, breply, NULL, &oid);
+
+	while(connectstr.isNull())
+	{
+		sw_discovery_read_socket(session);
+		kapp->eventLoop()->processEvents(QEventLoop::AllEvents);
+	}
+
+	KURL url(connectstr);
+
+	close();
+	KMessageBox::information(this, i18n("A GGZ server was found on %1.").arg(connectstr), i18n("Zeroconf success"));
+	emit signalConnect(url.host(), url.port(), input_name->text(), input_password->text(), m_loginmode);
+#endif
+}
+
