@@ -45,14 +45,21 @@
 #include <kpopupmenu.h>
 #include <kmenubar.h>
 #include <kstatusbar.h>
+#include <ksystemtray.h>
+#include <kiconloader.h>
+
+// Qt includes
+#include <qtooltip.h>
 
 KGGZBase::KGGZBase()
 : KMainWindow()
 {
-	int x, y, width, height;
+	int x, y, width, height, watcher;
 
+	tray = NULL;
 	m_about = NULL;
 	m_rooms = 0;
+	m_activity = 0;
 
 	KSimpleConfig konfig("kggzrc");
 	konfig.setGroup("kggzbase");
@@ -60,10 +67,15 @@ KGGZBase::KGGZBase()
 	y = konfig.readNumEntry("y");
 	width = konfig.readNumEntry("width");
 	height = konfig.readNumEntry("height");
+	watcher = konfig.readNumEntry("watcher");
 
-	statusBar()->insertItem(i18n("  Not connected  "), 1);
-	statusBar()->insertItem(i18n("  Loading...  "), 2);
-	statusBar()->insertItem(i18n("  No room selected  "), 3);
+	statusBar()->insertItem(i18n("Not connected"), STATUS_CONNECTION);
+	statusBar()->insertItem(i18n("Loading..."), STATUS_STATE);
+	statusBar()->insertItem(i18n("No room selected"), STATUS_ROOM);
+	statusBar()->insertItem(QString::null, STATUS_PLAYERS);
+	//statusBar()->insertItem(QString::null, STATUS_ENCRYPTION);
+	m_lock = new KStatusBarLabel(QString::null, STATUS_ENCRYPTION, statusBar());
+	statusBar()->addWidget(m_lock);
 
 	kggz = new KGGZ(this, "kggz");
 
@@ -73,6 +85,8 @@ KGGZBase::KGGZBase()
 	m_menu_ggz->insertSeparator();
 	m_menu_ggz->insertItem(kggzGetIcon(MENU_GGZ_MOTD), i18n("Message of the day"), MENU_GGZ_MOTD);
 	m_menu_ggz->insertSeparator();
+	m_menu_ggz->insertItem(kggzGetIcon(MENU_GGZ_WATCHER), i18n("Use panel watcher"), MENU_GGZ_WATCHER);
+	m_menu_ggz->insertSeparator();
 	m_menu_ggz->insertItem(kggzGetIcon(MENU_GGZ_STARTSERVER), i18n("Start server"), MENU_GGZ_STARTSERVER);
 	m_menu_ggz->insertItem(kggzGetIcon(MENU_GGZ_STOPSERVER), i18n("Stop server"), MENU_GGZ_STOPSERVER);
 	m_menu_ggz->insertSeparator();
@@ -82,6 +96,7 @@ KGGZBase::KGGZBase()
 	m_menu_ggz->setItemEnabled(MENU_GGZ_MOTD, FALSE);
 	if(KGGZCommon::findProcess("ggzd") > 0) m_menu_ggz->setItemEnabled(MENU_GGZ_STARTSERVER, FALSE);
 	else m_menu_ggz->setItemEnabled(MENU_GGZ_STOPSERVER, FALSE);
+	m_menu_ggz->setCheckable(true);
 
 	m_menu_client = new KPopupMenu(this, "menu_client");
 	m_menu_client->insertItem(kggzGetIcon(MENU_CLIENT_STARTUP), i18n("&Startup Screen"), MENU_CLIENT_STARTUP);
@@ -100,6 +115,7 @@ KGGZBase::KGGZBase()
 
 	m_menu_game = new KPopupMenu(this, "menu_games");
 	m_menu_game->insertItem(kggzGetIcon(MENU_GAME_INFO), i18n("&Information"), MENU_GAME_INFO);
+	m_menu_game->insertItem(kggzGetIcon(MENU_GAME_RULES), i18n("&Rules of the game"), MENU_GAME_RULES);
 	m_menu_game->insertSeparator();
 	m_menu_game->insertItem(kggzGetIcon(MENU_GAME_LAUNCH), i18n("&Launch new game"), MENU_GAME_LAUNCH);
 	m_menu_game->insertItem(kggzGetIcon(MENU_GAME_JOIN), i18n("&Join game"), MENU_GAME_JOIN);
@@ -117,7 +133,7 @@ KGGZBase::KGGZBase()
 	m_menu_preferences = new KPopupMenu(this, "menu_preferences");
 	m_menu_preferences->insertItem(kggzGetIcon(MENU_PREFERENCES_SETTINGS), i18n("Se&ttings"), MENU_PREFERENCES_SETTINGS);
 
-	helpMenu()->insertItem(QPixmap(KGGZ_DIRECTORY "/images/icons/ggz.png"), i18n("About the GGZ Gaming Zone"), 99, 5);
+	helpMenu()->insertItem(kggzGetIcon(MENU_HELP_GGZ), i18n("About the GGZ Gaming Zone"), 99, 5);
 	helpMenu()->connectItem(99, this, SLOT(slotAboutGGZ()));
 
 	menuBar()->insertItem(i18n("GG&Z"), m_menu_ggz, MENU_GGZ);
@@ -136,10 +152,13 @@ KGGZBase::KGGZBase()
 	connect(m_menu_game, SIGNAL(activated(int)), SLOT(slotMenu(int)));
 	connect(m_menu_preferences, SIGNAL(activated(int)), SLOT(slotMenu(int)));
 	connect(kggz, SIGNAL(signalMenu(int)), SLOT(slotMenuSignal(int)));
-	connect(kggz, SIGNAL(signalRoom(const char*, const char*)), SLOT(slotRoom(const char*, const char*)));
-	connect(kggz, SIGNAL(signalCaption(const char*)), SLOT(slotCaption(const char*)));
+	connect(kggz, SIGNAL(signalRoom(const char*, const char*, const char*, int)), SLOT(slotRoom(const char*, const char*, const char*, int)));
+	connect(kggz, SIGNAL(signalRoomChanged(const char*, const char*, int, int)), SLOT(slotRoomChanged(const char*, const char*, int, int)));
+	connect(kggz, SIGNAL(signalCaption(QString, bool)), SLOT(slotCaption(QString, bool)));
 	connect(kggz, SIGNAL(signalState(int)), SLOT(slotState(int)));
-	connect(kggz, SIGNAL(signalLocation(const char*)), SLOT(slotLocation(const char*)));
+	connect(kggz, SIGNAL(signalLocation(QString)), SLOT(slotLocation(QString)));
+	connect(kggz, SIGNAL(signalPlayers(int)), SLOT(slotPlayers(int)));
+	connect(kggz, SIGNAL(signalActivity(int)), SLOT(slotActivity(int)));
 
 	setCentralWidget(kggz);
 	setCaption(i18n("offline"));
@@ -147,18 +166,23 @@ KGGZBase::KGGZBase()
 	else setGeometry(KApplication::desktop()->width() / 2 - 250, KApplication::desktop()->height() / 2 - 225, 500, 441);
 	show();
 
+	if(watcher) slotMenu(MENU_GGZ_WATCHER);
+
 	kggz->menuView(KGGZ::VIEW_SPLASH);
 
-	statusBar()->changeItem(i18n("  Ready for connection.  "), 2);
+	statusBar()->changeItem(i18n("Ready for connection."), STATUS_STATE);
 }
 
 KGGZBase::~KGGZBase()
 {
 	KSimpleConfig konfig("kggzrc");
+	konfig.setGroup("kggzbase");
 	konfig.writeEntry("x", x());
 	konfig.writeEntry("y", y());
 	konfig.writeEntry("width", width());
 	konfig.writeEntry("height", height());
+	if(tray) konfig.writeEntry("watcher", 1);
+	else konfig.writeEntry("watcher", 0);
 }
 
 void KGGZBase::autoconnect(QString uri)
@@ -168,26 +192,67 @@ void KGGZBase::autoconnect(QString uri)
 
 QIconSet KGGZBase::kggzGetIcon(int menuid)
 {
-	const char *icon = NULL;
+	QString icon;
 	QIconSet iconset;
 
 	switch(menuid)
 	{
 		case MENU_GGZ_CONNECT:
-			icon = "connect.png";
+			return KGlobal::iconLoader()->loadIcon("connect_creating", KIcon::Small);
 			break;
 		case MENU_GGZ_DISCONNECT:
-			icon = "disconnect.png";
+			return KGlobal::iconLoader()->loadIcon("connect_no", KIcon::Small);
 			break;
+		case MENU_GGZ_MOTD:
+			return KGlobal::iconLoader()->loadIcon("news", KIcon::Small);
+			break;
+		case MENU_GGZ_WATCHER:
+			return KGlobal::iconLoader()->loadIcon("idea", KIcon::Small);
+			break;
+		case MENU_GGZ_QUIT:
+			return KGlobal::iconLoader()->loadIcon("exit", KIcon::Small);
+			break;
+		case MENU_PREFERENCES_SETTINGS:
+			return KGlobal::iconLoader()->loadIcon("configure", KIcon::Small);
+			break;
+		case MENU_GAME_INFO:
+			return KGlobal::iconLoader()->loadIcon("info", KIcon::Small);
+			break;
+		case MENU_GAME_RULES:
+			return KGlobal::iconLoader()->loadIcon("contents", KIcon::Small);
+			break;
+		case MENU_GAME_CANCEL:
+			return KGlobal::iconLoader()->loadIcon("cancel", KIcon::Small);
+			break;
+		case MENU_GAME_LAUNCH:
+			return KGlobal::iconLoader()->loadIcon("1uparrow", KIcon::Small);
+			break;
+		case MENU_GAME_JOIN:
+			return KGlobal::iconLoader()->loadIcon("1rightarrow", KIcon::Small);
+			break;
+		case MENU_HELP_GGZ:
+			return KGlobal::iconLoader()->loadIcon("ggz", KIcon::Small);
+			break;
+	}
+
+//(KGGZ_DIRECTORY "/images/icons/ggz.png")
+	switch(menuid)
+	{
+//		case MENU_GGZ_CONNECT:
+//			icon = "connect.png";
+//			break;
+//		case MENU_GGZ_DISCONNECT:
+//			icon = "disconnect.png";
+//			break;
 		case MENU_GGZ_STARTSERVER:
 			icon = "startserver.png";
 			break;
 		case MENU_GGZ_STOPSERVER:
 			icon = "stopserver.png";
 			break;
-		case MENU_GGZ_QUIT:
-			icon = "quit.png";
-			break;
+//		case MENU_GGZ_QUIT:
+//			icon = "quit.png";
+//			break;
 		case MENU_CLIENT_STARTUP:
 			icon = "startup.png";
 			break;
@@ -203,35 +268,35 @@ QIconSet KGGZBase::kggzGetIcon(int menuid)
 		case MENU_CLIENT_HELP:
 			icon = "browser.png";
 			break;
-		case MENU_GAME_INFO:
-			icon = "info.png";
-			break;
-		case MENU_GAME_LAUNCH:
-			icon = "launch.png";
-			break;
-		case MENU_GAME_JOIN:
-			icon = "join.png";
-			break;
-		case MENU_GAME_CANCEL:
-			icon = "cancel.png";
-			break;
+//		case MENU_GAME_INFO:
+//			icon = "info.png";
+//			break;
+//		case MENU_GAME_LAUNCH:
+//			icon = "launch.png";
+//			break;
+//		case MENU_GAME_JOIN:
+//			icon = "join.png";
+//			break;
+//		case MENU_GAME_CANCEL:
+//			icon = "cancel.png";
+//			break;
 		case MENU_GAME_UPDATE:
 			icon = "update.png";
 			break;
 		case MENU_GAME_GRUBBY:
 			icon = "grubby.png";
 			break;
-		case MENU_PREFERENCES_SETTINGS:
-		case MENU_PREFERENCES_PLAYERINFO:
-		case MENU_PREFERENCES_HOSTS:
-		case MENU_PREFERENCES_FTP:
-		case MENU_PREFERENCES_GAMES:
-		case MENU_PREFERENCES_THEMES:
-			icon = "pref.png";
-			break;
-		case MENU_PREFERENCES_PREFERENCES:
-			icon = "preferences.png";
-			break;
+//		case MENU_PREFERENCES_SETTINGS:
+//		case MENU_PREFERENCES_PLAYERINFO:
+//		case MENU_PREFERENCES_HOSTS:
+//		case MENU_PREFERENCES_FTP:
+//		case MENU_PREFERENCES_GAMES:
+//		case MENU_PREFERENCES_THEMES:
+//			icon = "pref.png";
+//			break;
+//		case MENU_PREFERENCES_PREFERENCES:
+//			icon = "preferences.png";
+//			break;
 		default:
 			icon = "unknown.png";
 			break;
@@ -239,10 +304,11 @@ QIconSet KGGZBase::kggzGetIcon(int menuid)
 
 	if(menuid >= MENU_ROOMS_SLOTS)
 	{
-		icon = "unknown.png";
+		icon = QString("games/%1.png").arg(m_lastgame); //"unknown.png";
+		KGGZDEBUG("gameicon: %s", icon.utf8().data());
 	}
 
-	iconset = QIconSet(QPixmap(QString(KGGZ_DIRECTORY "/images/icons/") + QString(icon)));
+	iconset = QIconSet(QPixmap(QString(KGGZ_DIRECTORY "/images/icons/") + icon));
 	return iconset;
 }
 
@@ -263,11 +329,34 @@ void KGGZBase::slotMenu(int id)
 			break;
 		case MENU_GGZ_DISCONNECT:
 			kggz->menuDisconnect();
-			statusBar()->changeItem(i18n("Not connected"), 1);
+			statusBar()->changeItem(i18n("Not connected"), STATUS_CONNECTION);
+			statusBar()->changeItem(QString::null, STATUS_PLAYERS);
+			m_lock->setFixedWidth(0);
 			break;
 		case MENU_GGZ_MOTD:
 			kggz->menuMotd();
 			break;
+		case MENU_GGZ_WATCHER:
+			if(m_menu_ggz->isItemChecked(MENU_GGZ_WATCHER))
+			{
+				if(tray)
+				{
+					m_menu_ggz->setItemChecked(MENU_GGZ_WATCHER, false);
+					delete tray;
+					tray = NULL;
+				}
+			}
+			else
+			{
+				if(!tray)
+				{
+					m_menu_ggz->setItemChecked(MENU_GGZ_WATCHER, true);
+					tray = new KSystemTray(this);
+					slotActivity(KGGZ::ACTIVITY_NONE);
+					tray->show();
+				}
+			}
+			break;
 		case MENU_GGZ_STARTSERVER:
 			kggz->menuServerLaunch();
 			break;
@@ -311,6 +400,9 @@ void KGGZBase::slotMenu(int id)
 		case MENU_GAME_INFO:
 			kggz->menuGameInfo();
 			break;
+		case MENU_GAME_RULES:
+			kggz->menuGameRules();
+			break;
 		case MENU_GAME_TEAM:
 			kggz->menuGameTeam();
 			break;
@@ -389,38 +481,104 @@ void KGGZBase::slotMenuSignal(int signal)
 			m_menu_game->setItemEnabled(MENU_GAME_SPECTATOR, FALSE);
 #endif
 			break;
+		case KGGZ::MENUSIG_RULES:
+			m_menu_game->setItemEnabled(MENU_GAME_RULES, TRUE);
+			break;
+		case KGGZ::MENUSIG_NORULES:
+			m_menu_game->setItemEnabled(MENU_GAME_RULES, FALSE);
+			break;
 		default:
 			KGGZDEBUG("Unknown signal for menu handling: %i!\n", signal);
 			break;
 	}
 }
 
-void KGGZBase::slotRoom(const char *roomname, const char *category)
+void KGGZBase::slotRoom(const char *roomname, const char *protocolname, const char *category, int numplayers)
 {
 	QString caption;
 
 #ifdef KGGZ_PATCH_C_AND_R
 	caption = QString("%1 (%2)").arg(roomname).arg(category);
 #else
+	Q_UNUSED(category);
+
 	caption = roomname;
+	if(numplayers > 0) caption = QString("%1 (%2)").arg(roomname).arg(numplayers);
 #endif
+	m_lastgame = protocolname;
 	m_menu_rooms->insertItem(kggzGetIcon(MENU_ROOMS_SLOTS + m_rooms), caption, MENU_ROOMS_SLOTS + m_rooms);
 	m_rooms++;
 }
 
-void KGGZBase::slotCaption(const char *caption)
+void KGGZBase::slotRoomChanged(const char *roomname, const char *protocolname, int roomnumber, int numplayers)
+{
+	QString caption;
+
+	caption = roomname;
+	if(numplayers > 0) caption = QString("%1 (%2)").arg(roomname).arg(numplayers);
+
+	m_lastgame = protocolname;
+	m_menu_rooms->changeItem(MENU_ROOMS_SLOTS + roomnumber, kggzGetIcon(MENU_ROOMS_SLOTS + roomnumber), caption);
+	KGGZDEBUG("***** ROOMS CHANGED: %s(%i)=%i", roomname, roomnumber, numplayers);
+}
+
+void KGGZBase::slotCaption(QString caption, bool encrypted)
 {
 	setCaption(caption);
-	statusBar()->changeItem(i18n("  Connected  "), 1);
+	statusBar()->changeItem(i18n("Connected"), STATUS_CONNECTION);
+
+	m_lock->setFixedSize(KIcon::SizeSmall, KIcon::SizeSmall);
+	if(encrypted)
+	{
+		//m_lock->setBackgroundColor(QColor(0, 255, 0));
+		QPixmap lock = KGlobal::iconLoader()->loadIcon("encrypted", KIcon::Small);
+		m_lock->setBackgroundPixmap(lock);
+		QToolTip::add(m_lock, i18n("Connection secured with TLS"));
+	}
+	else
+	{
+		//m_lock->setBackgroundColor(QColor(255, 0, 0));
+		QPixmap lock = KGlobal::iconLoader()->loadIcon("decrypted", KIcon::Small);
+		m_lock->setBackgroundPixmap(lock);
+		QToolTip::add(m_lock, i18n("Insecure connection"));
+	}
 }
 
 void KGGZBase::slotState(int state)
 {
-	statusBar()->changeItem(i18n("  State: ") + KGGZCommon::state((GGZStateID)state) + "  ", 2);
+	statusBar()->changeItem(i18n("State: ") + KGGZCommon::state((GGZStateID)state), STATUS_STATE);
 }
 
-void KGGZBase::slotLocation(const char *location)
+void KGGZBase::slotLocation(QString location)
 {
-	statusBar()->changeItem(QString("  ") + location + "  ", 3);
+	statusBar()->changeItem(location, STATUS_ROOM);
+}
+
+void KGGZBase::slotPlayers(int players)
+{
+	statusBar()->changeItem(QString("Players on server: %1").arg(players), STATUS_PLAYERS);
+}
+
+void KGGZBase::slotActivity(int activity)
+{
+	if((activity > KGGZ::ACTIVITY_NONE) && (activity <= m_activity)) return;
+
+	m_activity = activity;
+
+	if(tray)
+	{
+		if(activity == KGGZ::ACTIVITY_DIRECT)
+		{
+			tray->setPixmap(QPixmap(KGGZ_DIRECTORY "/images/icons/watcher_on.png"));
+		}
+		else if(activity == KGGZ::ACTIVITY_ROOM)
+		{
+			tray->setPixmap(QPixmap(KGGZ_DIRECTORY "/images/icons/watcher_room.png"));
+		}
+		else
+		{
+			tray->setPixmap(QPixmap(KGGZ_DIRECTORY "/images/icons/watcher_off.png"));
+		}
+	}
 }
 
